package ru.dbotthepony.mc.prng.mixin;

import net.minecraft.util.RandomSource;
import net.minecraft.world.level.levelgen.LegacyRandomSource;
import net.minecraft.world.level.levelgen.WorldgenRandom;
import net.minecraft.world.level.levelgen.XoroshiroRandomSource;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Overwrite;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import ru.dbotthepony.mc.prng.GJRAND64RandomSource;

@Mixin(WorldgenRandom.class)
public abstract class WorldgenRandomMixin {
	@Shadow
	protected RandomSource randomSource;

	// quite lazy solution because if someone genuinely want to get lcg they wont be able to
	@Inject(
		method = "<init>(Lnet/minecraft/util/RandomSource;)V",
		at = @At("TAIL")
	)
	public void constructorMix(RandomSource randomSource, CallbackInfo info) {
		if (randomSource instanceof LegacyRandomSource rng) {
			this.randomSource = new GJRAND64RandomSource(rng.seed.get());
		} else if (randomSource instanceof XoroshiroRandomSource rng) {
			this.randomSource = new GJRAND64RandomSource(rng.randomNumberGenerator.seedHi, rng.randomNumberGenerator.seedLo);
		}
	}

	@Overwrite
	public static RandomSource seedSlimeChunk(int chunkX, int chunkZ, long levelSeed, long salt) {
		return new LegacyRandomSource(
			levelSeed
				+ (long)(chunkX * chunkX * 4987142)
				+ (long)(chunkX * 5947611)
				+ (long)(chunkZ * chunkZ) * 4392871L
				+ (long)(chunkZ * 389711)
				^ salt
		);
	}
}
